@@ -15,12 +15,15 @@
 #include <sys/types.h>
 #include <sys/time.h>
 #include "fractint.h"
+#include "prototyp.h"
 
 int overflow = 0;
 
-char boxx[9216], boxy[4096];
-char boxvalues[2048];
+int boxx[2304], boxy[1024];
+int boxvalues[512];
 char tstack[4096];
+BYTE dacbox[256][3];
+BYTE olddacbox[256][3];
 
 extern int tabmode;
 
@@ -37,33 +40,35 @@ SEGTYPE extraseg=0;		/* extra 64K segment (allocated by init) */
 /* ********************** Mouse Support Variables ************************** */
 
 int lookatmouse=0;	/* see notes at mouseread routine */
-int savebase=0;		/* base clock ticks */ 
-int saveticks=0;	/* save after this many ticks */ 
+long savebase=0;		/* base clock ticks */ 
+long saveticks=0;	/* save after this many ticks */ 
 int finishrow=0;	/* save when this row is finished */
 
 int inside_help = 0;
 
 extern int slides;	/* 1 for playback */
 
-void toextra(tooffset, fromaddr, fromcount)
-int tooffset;
+unsigned int toextra(tooffset, fromaddr, fromcount)
+unsigned int tooffset;
 char *fromaddr;
 int fromcount;
 {
     bcopy(fromaddr,(char *)(extraseg+tooffset),fromcount);
+    return tooffset;
 }
 
-void fromextra(fromoffset, toaddr, tocount)
-int fromoffset;
+unsigned int fromextra(fromoffset, toaddr, tocount)
+unsigned int fromoffset;
 char *toaddr;
 int tocount;
 {
     bcopy((char *)(extraseg+fromoffset),toaddr,tocount);
+    return fromoffset;
 }
 
-int
+unsigned int
 cmpextra(cmpoffset,cmpaddr,cmpcount)
-int cmpoffset;
+unsigned int cmpoffset;
 char *cmpaddr;
 int cmpcount;
 {
@@ -179,12 +184,10 @@ keypressed() {
 	inside_help = 1;
 	help(0);
 	inside_help = 0;
-	restore_active_ovly();
 	return 0;
     } else if (ch==TAB && tabmode) {
 	keybuffer = 0;
 	tab_display();
-	restore_active_ovly();
 	return 0;
     }
     return ch;
@@ -316,7 +319,11 @@ int delaytime;
     static struct timeval delay;
     delay.tv_sec = delaytime/1000;
     delay.tv_usec = (delaytime%1000)*1000;
+#ifdef __SVR4
+    (void) select(0, (fd_set *) 0, (fd_set *) 0, (fd_set *) 0, &delay);
+#else
     (void) select(0, (int *) 0, (int *) 0, (int *) 0, &delay);
+#endif
 }
 
 /*
@@ -378,7 +385,9 @@ VOIDPTR addr;
     free((char *)addr);
 }
 
-erasesegment(segaddress,segvalue)
+void erasesegment(segaddress,segvalue)
+int segaddress;
+int segvalue;
 {
 }
 
@@ -420,11 +429,11 @@ char *a;
 }
 
 
-int
+void
 far_strcpy (a,b)
 char *a,*b;
 {
-    return (int)strcpy(a,b);
+    strcpy(a,b);
 }
 
 int
@@ -434,11 +443,11 @@ char *a, *b;
     return strcmp(a,b);
 }
 
-int
+void
 far_stricmp(a,b)
 char *a,*b;
 {
-    return stricmp(a,b);
+    stricmp(a,b);
 }
 
 int
@@ -449,29 +458,28 @@ int n;
     return strnicmp(a,b,n);
 }
 
-int
+void
 far_strcat (a,b)
 char *a,*b;
 {
-    return (int)strcat(a,b);
+    strcat(a,b);
 }
 
-int
+void
 far_memset ( a,c,len)
-VOIDPTR a;
+VOIDFARPTR a;
 int c;
 int len;
 {
-    return (int)memset(a,c,len);
+    memset(a,c,len);
 }
 
-int
+void
 far_memcpy ( a,b,len)
 VOIDFARPTR a,b;
 int len;
 {
     memcpy(a,b,len);
-    return (int)a;
 }
 
 int
@@ -482,12 +490,12 @@ int len;
     return memcmp(a,b,len);
 }
 
-int
+void
 far_memicmp(a,b,len)
 VOIDFARPTR a,b;
 int len;
 {
-    return memicmp(a,b,len);
+    memicmp(a,b,len);
 }
 
 /*
@@ -538,17 +546,18 @@ xmmdeallocate(a)
 unsigned int a;
 {}
 
-void
+unsigned int
 xmmmoveextended(a)
-int a;
+struct XMM_Move *a;
 {}
 
 /*
  * IIT coprocessor routines.
  */
 
+void
 load_mat(matrix)
-double matrix[16];
+double (*matrix)[4];
 {}
 
 void
@@ -599,7 +608,7 @@ decode_fractal_info(info,dir)
     }
 
     if (dir==1) {
-	strncpy(info->info_id,(const char *)bufPtr,8);
+	strncpy(info->info_id,(char *)bufPtr,8);
     } else {
 	strncpy((char *)bufPtr,info->info_id,8);
     }
@@ -775,7 +784,11 @@ static getLong(dst,src,dir)
 	(*src)[0] = (*dst)&0xff;
 	(*src)[1] = ((*dst)&0xff00)>>8;
 	(*src)[2] = ((*dst)&0xff0000)>>8;
+#ifdef __SVR4
+	(*src)[3] = (unsigned)((*dst)&0xff000000)>>8;
+#else
 	(*src)[3] = ((*dst)&0xff000000)>>8;
+#endif
     }
     (*src) += 4; /* sizeof(long) in MS_DOS */
 }
@@ -813,9 +826,16 @@ static getDouble(dst,src,dir)
 	if (i==8) {
 	    *dst = 0;
 	} else {
-	    e = (((*src)[7]&0x7f)<<4) + (((*src)[6]&0xf0)>>4) - 1023;
-	    f = 1 + ((*src)[6]&0x0f)/P4 + (*src)[5]/P12 + (*src)[4]/P20 +
-		(*src)[3]/P28 + (*src)[2]/P36 + (*src)[1]/P44 + (*src)[0]/P52;
+#ifdef __SVR4
+	    e = (((*src)[7]&0x7f)<<4) + ((int)((*src)[6]&0xf0)>>4) - 1023;
+	    f = 1 + (int)((*src)[6]&0x0f)/P4 + (int)((*src)[5])/P12 +
+        (int)((*src)[4])/P20 + (int)((*src)[3])/P28 + (int)((*src)[2])/P36 +
+        (int)((*src)[1])/P44 + (int)((*src)[0])/P52;
+#else
+      e = (((*src)[7]&0x7f)<<4) + (((*src)[6]&0xf0)>>4) - 1023;
+      f = 1 + ((*src)[6]&0x0f)/P4 + (*src)[5]/P12 + (*src)[4]/P20 +
+    (*src)[3]/P28 + (*src)[2]/P36 + (*src)[1]/P44 + (*src)[0]/P52;
+#endif
 	    f *= pow(2.,(double)e);
 	    if ((*src)[7]&0x80) {
 		f = -f;
@@ -881,8 +901,13 @@ static getFloat(dst,src,dir)
 	if (i==4) {
 	    *dst = 0;
 	} else {
-	    e = (((*src)[3]&0x7f)<<1)| (((*src)[2]&0x80)>>7) - 127;
-	    f = 1 + ((*src)[2]&0x7f)/P7 + (*src)[1]/P15 + (*src)[0]/P23;
+#ifdef __SVR4
+	    e = (((*src)[3]&0x7f)<<1)| ((int)((*src)[2]&0x80)>>7) - 127;
+	    f = 1 + (int)((*src)[2]&0x7f)/P7 + (int)((*src)[1])/P15 + (int)((*src)[0])/P23;
+#else
+      e = (((*src)[3]&0x7f)<<1)| (((*src)[2]&0x80)>>7) - 127;
+      f = 1 + ((*src)[2]&0x7f)/P7 + (*src)[1]/P15 + (*src)[0]/P23;
+#endif
 	    f *= pow(2.,(double)e);
 	    if ((*src)[3]&0x80) {
 		f = -f;
